@@ -4,7 +4,7 @@
 # strip projection, origin, orientation, epoch
 anonymize <- function(data,...)
 {
-  DROP <- class(data)=="telemetry"
+  DROP <- class(data)[1]=="telemetry"
   data <- listify(data)
 
   for(i in 1:length(data))
@@ -28,7 +28,7 @@ anonymize <- function(data,...)
 EPOCH <- as.POSIXct("1970-01-01 00:00.00 UTC",tz="GMT")
 pseudonymize <- function(data,center=c(0,0),datum="WGS84",origin="1111-11-11 11:11.11 UTC",tz="GMT",proj=NULL)
 {
-  DROP <- class(data)=="telemetry"
+  DROP <- class(data)[1]=="telemetry"
   data <- listify(data)
 
   if(is.null(proj)) { proj <- paste0("+proj=aeqd +lon_0=",center[1]," +lat_0=",center[2]," +datum=",datum) }
